package com.exner.tools.jkbikemechanicaldisasterprevention.ui

import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.exner.tools.jkbikemechanicaldisasterprevention.network.strava.StravaApiManager
import com.exner.tools.jkbikemechanicaldisasterprevention.preferences.StravaApiTokenStorage
import com.exner.tools.jkbikemechanicaldisasterprevention.preferences.UserPreferencesManager
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.theme.Theme
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.SharingStarted
import kotlinx.coroutines.flow.StateFlow
import kotlinx.coroutines.flow.stateIn
import kotlinx.coroutines.launch
import javax.inject.Inject

@HiltViewModel
class StravaLinkViewModel @Inject constructor(
    userPreferencesManager: UserPreferencesManager,
    private val stravaApiTokenStorage: StravaApiTokenStorage
) : ViewModel() {

    private val _isAuthenticationInitiated: MutableStateFlow<Boolean> = MutableStateFlow(false)
    val isAuthenticationInitiated: StateFlow<Boolean> = _isAuthenticationInitiated

    private val _isAuthenticated: MutableStateFlow<Boolean> = MutableStateFlow(false)

    val userSelectedTheme: StateFlow<Theme> = userPreferencesManager.theme().stateIn(
        scope = viewModelScope,
        started = SharingStarted.Companion.WhileSubscribed(stopTimeoutMillis = 5_000),
        initialValue = Theme.Auto
    )

    fun authenticateWithCode(code: String, key: String, secret: String) {
        if (!isAuthenticationInitiated.value) {
            _isAuthenticationInitiated.value = true
            if (code.isNotBlank()) {
                viewModelScope.launch {
                    val stravaApiManager = StravaApiManager(stravaApiTokenStorage)
                    stravaApiManager.retrieveTokens(
                        code = code,
                        key = key,
                        secret = secret
                    ) { stravaOAuthRequestResponse ->
                        if (stravaOAuthRequestResponse != null) {
                            _isAuthenticated.value = true
                        }
                    }
                }
            }
        }
    }
}