package com.exner.tools.jkbikemechanicaldisasterprevention.ui.jkbike

import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.exner.tools.jkbikemechanicaldisasterprevention.database.KJsRepository
import com.exner.tools.jkbikemechanicaldisasterprevention.database.entities.Activity
import com.exner.tools.jkbikemechanicaldisasterprevention.database.tools.WearLevel
import com.exner.tools.jkbikemechanicaldisasterprevention.preferences.UserPreferencesManager
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.launch
import kotlinx.datetime.Clock
import kotlinx.datetime.TimeZone
import kotlinx.datetime.todayIn
import javax.inject.Inject

@HiltViewModel
class HomeViewModel @Inject constructor(
    userPreferencesManager: UserPreferencesManager,
    private val repository: KJsRepository
) : ViewModel() {

    val observeActivityWithBikeData = repository.observeActivityWithBikeDataOrderedByDueDate

    val observeBikes = repository.observeBikes

    val stravaApiKey = userPreferencesManager.stravaApiKey()
    val stravaApiSecret = userPreferencesManager.stravaApiSecret()

    fun updateActivity(activity: Activity) {
        viewModelScope.launch {
            repository.updateActivity(activity)
        }
    }

    fun updateAttachedComponent(
        componentUid: Long,
        bikeUid: Long?
    ) {
        viewModelScope.launch {
            val bikeMileage = if (bikeUid != null) {
                repository.getBikeByUid(bikeUid)?.mileage ?: 0
            } else {
                0
            }
            val component = repository.getComponentByUid(componentUid)?.copy(
                lastCheckDate = Clock.System.todayIn(TimeZone.Companion.currentSystemDefault()),
            )
            val newComponent = if (bikeMileage > 0) {
                component?.copy(
                    lastCheckMileage = bikeMileage
                )
            } else {
                component
            }
            if (newComponent != null) {
                repository.updateComponent(newComponent)
            }
        }
    }

    fun logComponentWearLevel(componentUid: Long, wearLevel: WearLevel) {
        viewModelScope.launch {
            val component = repository.getComponentByUid(componentUid)
            if (component != null) {
                val updatedComponent = component.copy(
                    wearLevel = wearLevel
                )
                repository.updateComponent(updatedComponent)
            }
        }
    }
}