package com.exner.tools.jkbikemechanicaldisasterprevention.ui.jkbike

import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.exner.tools.jkbikemechanicaldisasterprevention.database.KJsRepository
import com.exner.tools.jkbikemechanicaldisasterprevention.database.tools.loadTemplateActivitiesJsonAndImport
import com.exner.tools.jkbikemechanicaldisasterprevention.preferences.StravaApiTokenStorage
import com.exner.tools.jkbikemechanicaldisasterprevention.preferences.UserPreferencesManager
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.theme.Theme
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.flow.SharingStarted
import kotlinx.coroutines.flow.StateFlow
import kotlinx.coroutines.flow.stateIn
import kotlinx.coroutines.launch
import javax.inject.Inject

enum class DistanceMeasure(val metric: String) {
    KM("km"),
    MI("mi")
}

@HiltViewModel
class SettingsViewModel @Inject constructor(
    private val userPreferencesManager: UserPreferencesManager,
    val repository: KJsRepository,
    private val stravaApiTokenStorage: StravaApiTokenStorage
) : ViewModel() {

    val userSelectedTheme: StateFlow<Theme> = userPreferencesManager.theme().stateIn(
        viewModelScope,
        SharingStarted.Companion.WhileSubscribed(),
        Theme.Auto
    )

    fun updateUserSelectedTheme(newTheme: Theme) {
        viewModelScope.launch {
            userPreferencesManager.setTheme(newTheme)
        }
    }

    val todoListsExpire: StateFlow<Boolean> = userPreferencesManager.todoListsExpire().stateIn(
        viewModelScope,
        SharingStarted.Companion.WhileSubscribed(),
        false
    )

    fun updateTodoListsExpire(newExpire: Boolean) {
        viewModelScope.launch {
            userPreferencesManager.setTodoListsExpire(newExpire)
        }
    }

    val distancesMetric: StateFlow<DistanceMeasure> = userPreferencesManager.distanceMeasure().stateIn(
        viewModelScope,
        SharingStarted.Companion.WhileSubscribed(),
        DistanceMeasure.KM
    )

    fun updateDistancesUseKm(newUseKm: DistanceMeasure) {
        viewModelScope.launch {
            userPreferencesManager.setDistanceMeasure(newUseKm)
        }
    }

    val stravaApiKey = userPreferencesManager.stravaApiKey()

    fun setStravaApiKey(key: String?) {
        viewModelScope.launch {
            userPreferencesManager.setStravaApiKey(key)
        }
    }

    val stravaApiSecret = userPreferencesManager.stravaApiSecret()

    fun setStravaApiSecret(secret: String?) {
        viewModelScope.launch {
            userPreferencesManager.setStravaApiSecret(secret)
        }
    }

    val isConnected = stravaApiTokenStorage.getStravaIsConnected()

    fun replaceTemplateActivitiesWithNewLanguage(language: String) {
        // check whether that language is OK
        loadTemplateActivitiesJsonAndImport(language) { newTemplateActivities ->
            viewModelScope.launch {
                // clear out templates
                repository.deleteAllTemplateActivities()
                // import List
                newTemplateActivities.forEach { templateActivity ->
                    repository.insertTemplateActivity(templateActivity)
                }
            }
        }
    }

}