package com.exner.tools.jkbikemechanicaldisasterprevention.ui.jkbike.bikes

import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.exner.tools.jkbikemechanicaldisasterprevention.database.KJsRepository
import com.exner.tools.jkbikemechanicaldisasterprevention.preferences.StravaApiTokenStorage
import com.exner.tools.jkbikemechanicaldisasterprevention.preferences.UserPreferencesManager
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.jkbike.DistanceMeasure
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.flow.SharingStarted
import kotlinx.coroutines.flow.StateFlow
import kotlinx.coroutines.flow.stateIn
import javax.inject.Inject

@HiltViewModel
class ManageBikesViewModel @Inject constructor(
    userPreferencesManager: UserPreferencesManager,
    repository: KJsRepository,
    private val stravaApiTokenStorage: StravaApiTokenStorage
) : ViewModel() {

    val bikes = repository.observeBikes

    val allBikes = repository.observeAllBikes

    val numberOfStravaConnectedBikes = repository.observeNumberOfStravaConnectedBikes

    val isConnected = stravaApiTokenStorage.getStravaIsConnected()

    val distanceMeasure: StateFlow<DistanceMeasure> = userPreferencesManager.distanceMeasure().stateIn(
        viewModelScope,
        SharingStarted.WhileSubscribed(),
        DistanceMeasure.KM
    )

    val stravaApiKey: StateFlow<String?> = userPreferencesManager.stravaApiKey().stateIn(
        viewModelScope,
        SharingStarted.WhileSubscribed(),
        null
    )
    val stravaApiSecret: StateFlow<String?> = userPreferencesManager.stravaApiSecret().stateIn(
        viewModelScope,
        SharingStarted.WhileSubscribed(),
        null
    )

}