package com.exner.tools.jkbikemechanicaldisasterprevention.ui.strava

import android.content.Intent
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.automirrored.filled.ArrowBack
import androidx.compose.material3.Button
import androidx.compose.material3.Text
import androidx.compose.material3.windowsizeclass.WindowSizeClass
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.ui.Modifier
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.res.stringResource
import androidx.core.net.toUri
import androidx.hilt.navigation.compose.hiltViewModel
import androidx.lifecycle.compose.collectAsStateWithLifecycle
import com.exner.tools.jkbikemechanicaldisasterprevention.R
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.KJsGlobalScaffoldViewModel
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.KJsResponsiveNavigation
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.customComponents.DefaultSpacer
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.customComponents.FullWidthButton
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.helpers.KJsAction
import com.ramcosta.composedestinations.annotation.Destination
import com.ramcosta.composedestinations.annotation.RootGraph
import com.ramcosta.composedestinations.generated.destinations.SettingsDestination
import com.ramcosta.composedestinations.generated.destinations.SyncWithStravaDestination
import com.ramcosta.composedestinations.navigation.DestinationsNavigator

@Destination<RootGraph>
@Composable
fun StravaApiCredentials(
    stravaApiCredentialsViewModel: StravaApiCredentialsViewModel = hiltViewModel(),
    kJsGlobalScaffoldViewModel: KJsGlobalScaffoldViewModel,
    destinationsNavigator: DestinationsNavigator,
    windowSizeClass: WindowSizeClass
) {
    kJsGlobalScaffoldViewModel.setDestinationTitle(stringResource(R.string.hdr_strava_api_credentials))

    val context = LocalContext.current

    val stravaApiKey by stravaApiCredentialsViewModel.stravaApiKey.collectAsStateWithLifecycle()
    val stravaApiSecret by stravaApiCredentialsViewModel.stravaApiSecret.collectAsStateWithLifecycle()

    KJsResponsiveNavigation(
        SettingsDestination,
        destinationsNavigator,
        windowSizeClass,
        myActions = listOf(
            KJsAction(
                imageVector = Icons.AutoMirrored.Filled.ArrowBack,
                contentDescription = stringResource(R.string.btn_desc_back),
                onClick = {
                    destinationsNavigator.navigateUp()
                }
            )
        ),
        headline = stringResource(R.string.hdr_strava_api_credentials)
    ) {
        // check whether there are credentials already
        if (stravaApiKey.isNullOrBlank() || stravaApiSecret.isNullOrBlank()) {
            // we need to get them!

            Column(
                modifier = Modifier
                    .fillMaxSize()
            ) {
                Text(text = "Please set up the Strava integration in Settings, first.")
                DefaultSpacer()
                FullWidthButton(onClick = {
                    destinationsNavigator.navigate(SettingsDestination)
                }) {
                    Text(text = stringResource(R.string.menu_item_settings))
                }
            }
        } else if (stravaApiCredentialsViewModel.isConnected) {
            Column(
                modifier = Modifier
                    .fillMaxSize()
            ) {
                Text(text = "Strava is connected. You can sync bikes.")
                DefaultSpacer()
                FullWidthButton(onClick = {
                    destinationsNavigator.navigate(SyncWithStravaDestination)
                }) {
                    Text(text = stringResource(R.string.dropdown_item_sync_with_strava))
                }
            }
        } else {
            Column(
                modifier = Modifier
                    .fillMaxSize()
            ) {
                Text(text = stringResource(R.string.you_have_strava_credentials))
                DefaultSpacer()
                // strava login
                Button(onClick = {
                    val intentUri = "https://www.strava.com/oauth/mobile/authorize".toUri()
                        .buildUpon()
                        .appendQueryParameter("client_id", stravaApiKey)
                        .appendQueryParameter("redirect_uri", "https://jkbike.net")
                        .appendQueryParameter("response_type", "code")
                        .appendQueryParameter("approval_prompt", "auto")
                        .appendQueryParameter("scope", "read,profile:read_all,activity:read_all")
                        .build()

                    val intent = Intent(Intent.ACTION_VIEW, intentUri)
                    context.startActivity(intent)
                }) {
                    Text(text = stringResource(R.string.connect_with_strava))
                }
                DefaultSpacer()
                Text(text = "You can also change the Strava integration in Settings.")
                DefaultSpacer()
                FullWidthButton(onClick = {
                    destinationsNavigator.navigate(SettingsDestination)
                }) {
                    Text(text = stringResource(R.string.menu_item_settings))
                }
            }
        }
    }
}
